@@ -30,6 +30,7 @@
 class QModelIndex;
 class QWebView;
 class QtHelpProviderAbstract;
+class QTemporaryFile;
 
 class QtHelpDocumentation : public QObject, public KDevelop::IDocumentation
 {
@@ -49,15 +50,23 @@ class QtHelpDocumentation : public QObject, public KDevelop::IDocumentation
         QMap<QString, QUrl> info() const { return m_info; }
 
         static QtHelpProviderAbstract* s_provider;
+
+    public slots:
+        void viewContextMenuRequested(const QPoint& pos);
+
     private slots:
         void jumpedTo(const QUrl& newUrl);
 
+    private:
+        void setUserStyleSheet(QWebView* view, const QUrl& url);
+
     private:
         QtHelpProviderAbstract *m_provider;
         const QString m_name;
         const QMap<QString, QUrl> m_info;
         const QMap<QString, QUrl>::const_iterator m_current;
         QWebView* lastView;
+        QWeakPointer<QTemporaryFile> m_lastStyleSheet;
 };
 
 class HomeDocumentation : public QObject, public KDevelop::IDocumentation
