@@ -43,7 +43,7 @@
 using namespace Cpp;
 using namespace KDevelop;
 
-const int maxParentDepth = 20;
+const uint maxParentDepth = 20;
 
 namespace Cpp {
 
@@ -55,7 +55,7 @@ KDEVCPPDUCHAIN_EXPORT  QList<KDevelop::Declaration*> findLocalDeclarations( KDev
     return ret;
   }
   
-  ret += context->findLocalDeclarations( identifier, SimpleCursor::invalid(), topContext );
+  ret += context->findLocalDeclarations( identifier, CursorInRevision::invalid(), topContext );
   if( !ret.isEmpty() )
     return ret;
 
@@ -94,7 +94,7 @@ uint buildIdentifierForType(AbstractType::Ptr type, IndexedTypeIdentifier& id, u
     uint maxPointerLevel = buildIdentifierForType(pointerType->baseType(), id, pointerLevel, top);
     if(type->modifiers() & AbstractType::ConstModifier)
       id.setIsConstPointer(maxPointerLevel - pointerLevel, true);
-    if(id.pointerDepth() < pointerLevel)
+    if(static_cast<uint>(id.pointerDepth()) < pointerLevel)
       id.setPointerDepth(pointerLevel);
     
     return maxPointerLevel;
@@ -130,25 +130,44 @@ IndexedTypeIdentifier identifierForType(AbstractType::Ptr type, TopDUContext* to
   return ret;
 }
 
-QList< QPair<Declaration*, int> > hideOverloadedDeclarations( const QList< QPair<Declaration*, int> >& declarations ) {
+QList< QPair<Declaration*, int> > hideOverloadedDeclarations( const QList< QPair<Declaration*, int> >& declarations,
+                                                              bool preferConst )
+{
   QHash<Identifier, Declaration*> nearestDeclaration;
   QHash<Declaration*, int> depthHash;
   QSet<Identifier> hadNonForwardDeclaration; //Set of all non function-declarations that had a declaration that was not a forward-declaration.
+  // Set of function-declarations with a const overload
+  QSet<IndexedIdentifier> hasConstOverload;
 
   typedef QPair<Declaration*, int> Pair;
   foreach(  const Pair& decl, declarations ) {
     depthHash[decl.first] = decl.second;
-    
+
     QHash<Identifier, Declaration*>::iterator it = nearestDeclaration.find(decl.first->identifier());
 
+    bool prefer = false;
     if(it == nearestDeclaration.end()) {
-      nearestDeclaration[ decl.first->identifier() ] = decl.first;
-    }else if(decl.first->isForwardDeclaration() == (*it)->isForwardDeclaration() || (!decl.first->isForwardDeclaration() && (*it)->isForwardDeclaration())){
+      prefer = true;
+    } else if((!decl.first->isForwardDeclaration() && (*it)->isForwardDeclaration())) {
       //Always prefer non forward-declarations over forward-declarations
-      if((!decl.first->isForwardDeclaration() && (*it)->isForwardDeclaration()) || decl.second < depthHash[*it])
-        nearestDeclaration[ decl.first->identifier() ] = decl.first;
+      prefer = true;
+    } else if(decl.first->isForwardDeclaration() == (*it)->isForwardDeclaration()) {
+      const int depth = depthHash.value(*it);
+      if (decl.second < depth) {
+        prefer = true;
+      } else if (decl.second == depth && decl.first->isFunctionDeclaration()
+                && decl.first->context()->type() == DUContext::Class
+                && TypeUtils::isConstant((*it)->abstractType()) != TypeUtils::isConstant(decl.first->abstractType()))
+      {
+        // const overload
+        hasConstOverload << decl.first->indexedIdentifier();
+      }
     }
-    
+
+    if(prefer) {
+      nearestDeclaration[ decl.first->identifier() ] = decl.first;
+    }
+
     if(!decl.first->isForwardDeclaration() && !decl.first->isFunctionDeclaration())
       hadNonForwardDeclaration.insert(decl.first->identifier());
   }
@@ -158,6 +177,20 @@ QList< QPair<Declaration*, int> > hideOverloadedDeclarations( const QList< QPair
   ///Only keep the declarations of each name on the lowest inheritance-level, or that are not forward-declarations
   foreach( const Pair& decl, declarations ) {
     if( depthHash[nearestDeclaration[decl.first->identifier()]] == decl.second ) {
+      if(decl.first->isFunctionDeclaration() && decl.first->context()->type() == DUContext::Class)
+      {
+        // hide non-const methods if preferConst is set, or vice-versa
+        const bool isConst = TypeUtils::isConstant(decl.first->abstractType());
+        const bool hasOverload = hasConstOverload.contains(decl.first->indexedIdentifier());
+        if (hasOverload && isConst != preferConst) {
+          // if we have an overload, always prefer the version with correct constness
+          continue;
+        } else if (preferConst && !isConst) {
+          // otherwise only hide non-cost methods if we are looking for const methods
+          // const-methods should be shown when we are looking for non-const methods though
+          continue;
+        }
+      }
       if(decl.first->isFunctionDeclaration() || !decl.first->isForwardDeclaration() || !hadNonForwardDeclaration.contains(decl.first->identifier()))
         ret << decl;
     }
@@ -166,7 +199,7 @@ QList< QPair<Declaration*, int> > hideOverloadedDeclarations( const QList< QPair
   return ret;
 }
 
-QList<KDevelop::Declaration*> findDeclarationsSameLevel(KDevelop::DUContext* context, const Identifier& identifier, const KDevelop::SimpleCursor& position)
+QList<KDevelop::Declaration*> findDeclarationsSameLevel(KDevelop::DUContext* context, const Identifier& identifier, const KDevelop::CursorInRevision& position)
 {
   if( context->type() == DUContext::Namespace || context->type() == DUContext::Global ) {
     ///May have been forward-declared anywhere
@@ -219,7 +252,7 @@ KDevelop::QualifiedIdentifier namespaceScopeComponentFromContext(QualifiedIdenti
   return prefix;
 }
 
-Declaration* localClassFromCodeContext(DUContext* context)
+Declaration* localClassFromCodeContext(const DUContext* context)
 {
   if(!context)
     return 0;
@@ -263,7 +296,7 @@ Declaration* localClassFromCodeContext(DUContext* context)
   return 0;
 }
 
-Declaration* localFunctionFromCodeContext(DUContext* context)
+Declaration* localFunctionFromCodeContext(const DUContext* context)
 {
   while( context->parentContext() && context->type() == DUContext::Other && context->parentContext()->type() == DUContext::Other )
   { //Move context to the top context of type "Other". This is needed because every compound-statement creates a new sub-context.
@@ -381,9 +414,6 @@ KDevelop::DUContext* logicalParentContext(KDevelop::DUContext* context, KDevelop
  * Preprocess the given string using the macros from given EnvironmentFile up to the given line
  * If line is -1, all macros are respected.
  * This is a quite slow operation, because thousands of macros need to be shuffled around.
- * 
- * @todo maybe implement a version of rpp::Environment that directly works on EnvironmentFile,
- * without needing to copy all macros.
  * */
 QString preprocess( const QString& text, Cpp::EnvironmentFile* file, int line, QSet<IndexedString> disableMacros ) {
 
@@ -618,11 +648,10 @@ AbstractType::Ptr stripType(KDevelop::AbstractType::Ptr type, DUContext* ctx) {
           currentId.clearTemplateIdentifiers();
           
           KDevelop::InstantiationInformation instantiationInfo = tempDecl->instantiatedWith().information();
-          int neededParameters = 0;
           KDevelop::InstantiationInformation newInformation(instantiationInfo);
           newInformation.templateParametersList().clear();
           
-          for(neededParameters = 0; neededParameters < instantiationInfo.templateParametersSize(); ++neededParameters) {
+          for(uint neededParameters = 0; neededParameters < instantiationInfo.templateParametersSize(); ++neededParameters) {
             newInformation.templateParametersList().append(instantiationInfo.templateParameters()[neededParameters]);
             AbstractType::Ptr niceParam = stripType(instantiationInfo.templateParameters()[neededParameters].abstractType(), ctx);
             if(niceParam) {
@@ -640,18 +669,17 @@ AbstractType::Ptr stripType(KDevelop::AbstractType::Ptr type, DUContext* ctx) {
         }else{
           newTypeName = decl->qualifiedIdentifier();
         }
-
         //Strip unneded prefixes of the scope
-        while(newTypeName.count() > 1) {
-          KDevelop::QualifiedIdentifier candidate = newTypeName.mid(1);
+        KDevelop::QualifiedIdentifier candidate = newTypeName;
+        while(candidate.count() > 1) {
+          candidate = candidate.mid(1);
           QList< KDevelop::Declaration* > decls = ctx->findDeclarations(candidate);
           if(decls.isEmpty())
-            break;
-          if(decls[0]->kind() != Declaration::Type || decls[0]->indexedType() != type->indexed())
+            continue; // type aliases might be available for nested sub scopes, hence we must not break early
+          if(decls[0]->kind() != Declaration::Type || TypeUtils::removeConstModifier(decls[0]->indexedType()) != TypeUtils::removeConstModifier(type->indexed()))
             break;
           newTypeName = candidate;
         }
-        
         if(newTypeName == decl->qualifiedIdentifier())
           return type;
         
@@ -706,11 +734,9 @@ IndexedTypeIdentifier shortenedTypeIdentifier(AbstractType::Ptr type, DUContext*
 {
 
   bool isReference = false;
-  bool isConstReference = false;
-  if(type.cast<ReferenceType>()) {
+  if(ReferenceType::Ptr refType = type.cast<ReferenceType>()) {
     isReference = true;
-    isConstReference = type.cast<ReferenceType>()->modifiers() & AbstractType::ConstModifier;
-    type = type.cast<ReferenceType>()->baseType();
+    type = refType->baseType();
   }
 
   type = shortenTypeForViewing(type);
@@ -726,11 +752,8 @@ IndexedTypeIdentifier shortenedTypeIdentifier(AbstractType::Ptr type, DUContext*
     identifier = type.cast<DelayedType>()->identifier();
   identifier = stripPrefixIdentifiers(identifier, stripPrefix);
 
-  if(isReference) {
+  if(isReference)
     identifier.setIsReference(true);
-    if(isConstReference)
-      identifier.setIsConstant(true);
-  }
   
 //   if(identifier.toString().length() > desiredLength)
 //     identifier = Cpp::unTypedefType(decl, identifier);
@@ -798,33 +821,42 @@ QualifiedIdentifier stripPrefixes(DUContext* ctx, QualifiedIdentifier id)
 {
   if(!ctx)
     return id;
-  
-  QualifiedIdentifier basicId = id;
-  
+
   QList<QualifiedIdentifier> imports = ctx->fullyApplyAliases(QualifiedIdentifier(), ctx->topContext());
   if(imports.contains(id))
     return QualifiedIdentifier(); ///The id is a namespace that is imported into the current context
   
-  QList< Declaration* > basicDecls = ctx->findDeclarations(id, SimpleCursor::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
+  QList< Declaration* > basicDecls = ctx->findDeclarations(id, CursorInRevision::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
   
   if(basicDecls.isEmpty())
     return id;
   
-  while(!id.isEmpty())
+  QualifiedIdentifier newId = id.mid(1);
+  while(!newId.isEmpty())
   {
-    QualifiedIdentifier newId = id.mid(1);
-    QList< Declaration* > foundDecls = ctx->findDeclarations(newId, SimpleCursor::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
-    
+    QList< Declaration* > foundDecls = ctx->findDeclarations(newId, CursorInRevision::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
+
     if(foundDecls == basicDecls)
-      id = newId;
-    else
-      break;
+      id = newId; // must continue to find the shortest possible identifier
+                  // esp. for cases where nested namespaces are used (e.g. using namespace a::b::c;)
+
+    newId = newId.mid(1);
   }
 
   return id;
 }
 
-KDevelop::IndexedIdentifier castIdentifier(Identifier("operator{...cast...}"));
-KDevelop::IndexedIdentifier unnamedNamespaceIdentifier(Identifier("unnamed{...namespace...}"));
+const KDevelop::IndexedIdentifier& castIdentifier()
+{
+  static const KDevelop::IndexedIdentifier id(Identifier("operator{...cast...}"));
+  return id;
+}
+
+const KDevelop::IndexedIdentifier& unnamedNamespaceIdentifier()
+{
+  static const KDevelop::IndexedIdentifier id(Identifier("unnamed{...namespace...}"));
+  return id;
+}
+
 }
 
