@@ -39,6 +39,7 @@
 #include "expressionparser.h"
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/duchainregister.h>
+#include <util/pushvalue.h>
 #include <name_compiler.h>
 #include <parsesession.h>
 #include <rpp/chartools.h>
@@ -111,6 +112,7 @@ AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::
     ReferenceType::Ptr newRet( new ReferenceType() );
     newRet->setModifiers(modifiers);
     newRet->setBaseType( ret );
+    newRet->setIsRValue(id.isRValue());
     ret = newRet.cast<AbstractType>();
   }
 
@@ -242,10 +244,16 @@ namespace Cpp {
 //   return true;
 // }
 
-TemplateDeclaration::TemplateDeclaration(const TemplateDeclaration& /*rhs*/) : m_instantiatedFrom(0) {
+TemplateDeclaration::TemplateDeclaration(const TemplateDeclaration& /*rhs*/)
+: m_instantiatedFrom(0)
+, m_instantiationDepth(0)
+{
 }
 
-TemplateDeclaration::TemplateDeclaration() : m_instantiatedFrom(0) {
+TemplateDeclaration::TemplateDeclaration()
+: m_instantiatedFrom(0)
+, m_instantiationDepth(0)
+{
 }
 
 Declaration* TemplateDeclaration::specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance) {
@@ -326,8 +334,8 @@ void TemplateDeclaration::setSpecializedFrom(TemplateDeclaration* other) {
   IndexedDeclaration indexedOther(dynamic_cast<Declaration*>(other));
   Q_ASSERT(indexedSelf.data());
   
-  if( specializedFrom().data() )
-    dynamic_cast<TemplateDeclaration*>(specializedFrom().data())->removeSpecializationInternal(indexedSelf);
+  if( TemplateDeclaration* tplDec = dynamic_cast<TemplateDeclaration*>(specializedFrom().data()) )
+    tplDec->removeSpecializationInternal(indexedSelf);
 
   setSpecializedFromInternal(indexedOther);
 
@@ -470,7 +478,6 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
     ///Specialize involved contexts
     Q_ASSERT(context->parentContext()); //Top-context is not allowed
     contextCopy = new StandardCppDUContext(context->range(), parentContext, true); //We do not need to care about TopDUContext here, because a top-context can not be instantiated
-    contextCopy->setSmartRange(context->smartRange(), KDevelop::DocumentRangeObject::DontOwn); //The range belongs to the original context, so flag it not to be owned by the context
     contextCopy->setType(context->type());
     contextCopy->setLocalScopeIdentifier(context->localScopeIdentifier());
 
@@ -534,7 +541,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
       if( import->type() == KDevelop::DUContext::Template || import->type() == KDevelop::DUContext::Function )
       {
         DUContext* ctx = import->instantiate(templateArguments, source);
-        contextCopy->addImportedParentContext( ctx, SimpleCursor(), true );
+        contextCopy->addImportedParentContext( ctx, CursorInRevision(), true );
 
         if( instantiatedDeclaration && import->type() == KDevelop::DUContext::Template ) {
           TemplateDeclaration* tempDecl = dynamic_cast<TemplateDeclaration*>(instantiatedDeclaration);
@@ -547,7 +554,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
       else
       {
         //Import all other imported contexts
-        contextCopy->addImportedParentContext( import, SimpleCursor::invalid(), true );
+        contextCopy->addImportedParentContext( import, CursorInRevision::invalid(), true );
       }
     }
 
@@ -568,7 +575,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
             {
               if( baseClass->declaration(source) && baseClass->declaration(source)->internalContext() )
               {
-                contextCopy->addImportedParentContext( baseClass->declaration(source)->internalContext(), SimpleCursor::invalid(), true );
+                contextCopy->addImportedParentContext( baseClass->declaration(source)->internalContext(), CursorInRevision::invalid(), true );
               }
               BaseClassInstance newInstance(base);
               newInstance.baseClass = newType->indexed();
@@ -807,7 +814,7 @@ QPair<unsigned int, TemplateDeclaration*> TemplateDeclaration::matchTemplatePara
     //Use the information from the specialization-information to build the template-identifiers
     for(uint a = 0; a < instantiationSpecializedWith.templateParametersSize(); ++a) {
       if(instantiationSpecializedWith.templateParameters()[a] != info.templateParameters()[a]) {
-        ifDebugMatching( kDebug() << "mismatch in final parameters at" << a << instantiationSpecializedWith.templateParameters()[a].type()->toString() << info.templateParameters()[a].type()->toString(); )
+        ifDebugMatching( kDebug() << "mismatch in final parameters at" << a << instantiationSpecializedWith.templateParameters()[a].abstractType()->toString() << info.templateParameters()[a].abstractType()->toString(); )
         return qMakePair(0u, (TemplateDeclaration*)0);
       }
     }
@@ -819,7 +826,6 @@ QPair<unsigned int, TemplateDeclaration*> TemplateDeclaration::matchTemplatePara
 Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _templateArguments, const TopDUContext* source, bool forceLocal )
 {
   InstantiationInformation templateArguments(_templateArguments);
-  
 /*  if(dynamic_cast<TopDUContext*>(dynamic_cast<const Declaration*>(this)->context())) {
     Q_ASSERT(templateArguments.previousInstantiationInformation == 0);
   }*/
@@ -852,6 +858,12 @@ Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _
   if(!source)
     return 0;
   
+  if (m_instantiationDepth > 5) {
+      kWarning() << "depth-limit reached while instantiating template declaration with" << _templateArguments.toString();
+      return 0;
+  }
+  PushValue<int> depthCounter(m_instantiationDepth, m_instantiationDepth + 1);
+
   DUContext* surroundingContext = dynamic_cast<const Declaration*>(this)->context();
   if(!surroundingContext) {
     kDebug() << "Declaration has no context:" << dynamic_cast<Declaration*>(this)->qualifiedIdentifier().toString() << dynamic_cast<Declaration*>(this)->toString();
@@ -870,7 +882,7 @@ Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _
 
   if(!forceLocal) {
     ///Apply default-parameters
-    if(templateContext && (templateContext->localDeclarations().count() > templateArguments.templateParametersSize() || (templateArguments.templateParametersSize() == 1 && !templateArguments.templateParameters()[0].isValid() && templateContext->localDeclarations().count() >= 1))) {
+    if(templateContext && (static_cast<uint>(templateContext->localDeclarations().count()) > templateArguments.templateParametersSize() || (templateArguments.templateParametersSize() == 1 && !templateArguments.templateParameters()[0].isValid() && templateContext->localDeclarations().count() >= 1))) {
       //Do a little fake instantiation so all the default-parameters are correctly applied right here. Then, they can be used
       //during specialization-matching.
       DUContext* newTemplateContext = instantiateDeclarationAndContext( surroundingContext, source, templateContext, templateArguments, 0, 0, true );
